package tui

import (
	"errors"
	"fmt"
	"testing"

	"git.tordarus.net/Tordarus/tui"
	"git.tordarus.net/Tordarus/tui/views"
	"github.com/gdamore/tcell"
)

func TestFlowLayout(t *testing.T) {
	textView := views.NewTextView("hello world!")
	textView.SetStyle(tui.StyleDefault.Background(tcell.ColorRed).Foreground(tcell.ColorBlack))

	marginView := views.NewMarginView(textView)
	marginView.SetMargin(3, 1, 1, 0)

	//borderView := views.NewBorderView(textView)

	textView2 := views.NewTextView("Hi!")
	textView2.SetStyle(tui.StyleDefault.Background(tcell.ColorBlue).Foreground(tcell.ColorYellow))

	growView := views.NewGrowView()
	growView.SetStyle(tui.StyleDefault.Background(tcell.ColorGreen))

	growView2 := views.NewGrowView()
	growView2.SetStyle(tui.StyleDefault.Background(tcell.ColorYellow))

	flowLayout := views.NewFlowLayout(tui.Vertical)
	flowLayout.AppendViews(marginView, growView, textView2)

	constrainView := views.NewConstrainView(flowLayout)
	constrainView.SetStyle(tui.StyleDefault.Background(tcell.ColorPurple))
	constrainView.Constrain(-1, -1)

	screen, err := tui.NewScreen(constrainView)
	if err != nil {
		t.Error(err)
		return
	}

	screen.KeyPressed = func(event *tui.KeyEvent) (consumed bool) {
		textView.Text = event.When().String()

		if event.Key() == tcell.KeyCtrlC {
			screen.StopWithError(errors.New(fmt.Sprintf("key: %#v | rune: %s", event.Key(), string(event.Rune()))))
		}

		return true
	}

	err = screen.Start()
	fmt.Println(err)
}

func TestSeparatorLayout(t *testing.T) {
	textView := views.NewTextView("hello world!")
	textView.SetStyle(tui.StyleDefault.Background(tcell.ColorRed).Foreground(tcell.ColorBlack))

	frameView := views.NewFrameView(textView)

	textView2 := views.NewTextView("Hi!")
	textView2.SetStyle(tui.StyleDefault.Background(tcell.ColorBlue).Foreground(tcell.ColorYellow))

	growView := views.NewGrowView()
	growView.SetStyle(tui.StyleDefault.Background(tcell.ColorGreen))

	growView2 := views.NewGrowView()
	growView2.SetStyle(tui.StyleDefault.Background(tcell.ColorYellow))

	separatorLayout := views.NewSeparatorLayout(tui.Vertical)
	separatorLayout.AppendView(frameView, 1)
	separatorLayout.AppendView(growView, 1)
	separatorLayout.AppendView(textView2, 1)

	screen, err := tui.NewScreen(separatorLayout)
	if err != nil {
		t.Error(err)
		return
	}

	screen.KeyPressed = func(event *tui.KeyEvent) (consumed bool) {
		textView.Text = event.When().String()

		if event.Key() == tcell.KeyCtrlC {
			screen.StopWithError(errors.New(fmt.Sprintf("key: %#v | rune: %s", event.Key(), string(event.Rune()))))
		}

		return true
	}

	err = screen.Start()
	fmt.Println(err)
}

func TestBorderLayout(t *testing.T) {
	topView := views.NewConstrainView(nil)
	topView.SetStyle(tui.StyleDefault.Background(tcell.ColorBlue))
	topView.Constrain(10, 10)

	bottomView := views.NewConstrainView(nil)
	bottomView.SetStyle(tui.StyleDefault.Background(tcell.ColorRed))
	bottomView.Constrain(10, 10)

	leftView := views.NewConstrainView(nil)
	leftView.SetStyle(tui.StyleDefault.Background(tcell.ColorYellow))
	leftView.Constrain(10, 10)

	rightView := views.NewConstrainView(nil)
	rightView.SetStyle(tui.StyleDefault.Background(tcell.ColorGreen))
	rightView.Constrain(10, 10)

	centerView := views.NewConstrainView(nil)
	centerView.SetStyle(tui.StyleDefault.Background(tcell.ColorPurple))
	centerView.Constrain(10, 10)

	borderLayout := views.NewBorderLayout()
	borderLayout.SetStyle(tui.StyleDefault.Background(tcell.ColorPurple))
	borderLayout.SetView(topView, views.Top)
	borderLayout.SetView(bottomView, views.Bottom)
	borderLayout.SetView(leftView, views.Left)
	borderLayout.SetView(rightView, views.Right)
	borderLayout.SetView(centerView, views.Center)

	screen, err := tui.NewScreen(borderLayout)
	if err != nil {
		t.Error(err)
		return
	}

	screen.KeyPressed = func(event *tui.KeyEvent) (consumed bool) {
		if event.Key() == tcell.KeyCtrlC {
			screen.StopWithError(errors.New(fmt.Sprintf("key: %#v | rune: %s", event.Key(), string(event.Rune()))))
		}
		return true
	}

	err = screen.Start()
	fmt.Println(err)
}
